package rt.lanyo.game.ddz;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import rt.lanyo.game.ddz.mgr.ImgMgr;
import rt.lanyo.game.ddz.mgr.OnlineMgr;
import rt.lanyo.game.ddz.mgr.PlayerMgr;
import rt.lanyo.game.ddz.mgr.SaveMgr;
import rt.lanyo.game.ddz.mgr.TaskMgr;
import rt.lanyo.game.ddz.model.Card;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.service.MusicService;
import rt.lanyo.game.ddz.util.G;
import rt.lanyo.game.ddz.util.J;
import rt.lanyo.game.ddz.util.S;
import rt.lanyo.game.ddz.view.GameView;
import rt.lanyo.game.ddz.vo.CmdVO;
import rt.lanyo.game.ddz.widget.MyDialog;
import android.app.Dialog;
import android.graphics.drawable.BitmapDrawable;
import android.os.Bundle;
import android.os.Message;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnTouchListener;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.PopupWindow;
import android.widget.RadioButton;
import android.widget.SimpleAdapter;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.PopupWindow.OnDismissListener;

import com.google.gson.reflect.TypeToken;

public class Game extends Base implements OnDismissListener, OnTouchListener{
	public static final byte DIALOG_SUCCESS = 1;
	public static final byte DIALOG_FAILED = 2;
	public static final byte DIALOG_OUT = 3;
	public static final byte DIALOG_FAILED_OUT = 4;
	public static final byte DIALOG_SETTING = 5;
	public static final byte DIALOG_TASK = 6;
	public static final byte DIALOG_INFO = 8;
	
	public static final byte START_ONLINE_GAME = 7; //用户开始游戏，要隐藏按钮
	public static final byte SHOW_READY = 10;  //显示准备按钮
	
	private GameView gv;
	private PopupWindow scoreBoard;
	private byte what;
	private ListView lv;
	private SimpleAdapter adapter;
	
	private Button more;//桌面上的按钮
	private Button exit;
	private Button backBoard;
	private Button ready;
	
	private MyDialog exitDialog;//退出提醒对话框
	private MyDialog successDialog;//闯关成功对话框
	private MyDialog failedDialog;//闯关失败对话框
	private MyDialog outDialog;//闯关出局对话框
	private MyDialog failedOutDialog;//闯关失败而且积分不够
	private MyDialog taskTipDialog;//任务提示
	private MyDialog infoDialog;
	public static Player clickPlayer; //被点击的player
	
	public static boolean exitGame = false;
	public static boolean levelUp = false;
	
	private Type joinType = new TypeToken<Map<Byte, Player>>(){}.getType();
	private Type cardType = new TypeToken<Map<Integer, List<Integer>>>(){}.getType();
	private Type cardType2 = new TypeToken<List<Integer>>(){}.getType();
	
	protected void create() {
		exitGame = false;
		setContentView(R.layout.game);
		gv = (GameView)findViewById(R.id.game_view);
		
		initWidgets();
	}
	
	protected void handle(Message msg) {
		super.handle(msg);
		what = (byte) msg.what;  //这个变量在闯关任务中有用
		switch(what) {
		case DIALOG_SETTING:
			openSettingDialog();
			break;
		case DIALOG_TASK:
			String taskContent = TaskMgr.getTaskContent(G.me);
			taskTipDialog = MyDialog.create(TaskMgr.getIntroStr(G.me.taskId), taskContent, R.string.sure);
			break;
		case START_ONLINE_GAME:
			//发送到服务器说取消准备
			OnlineMgr.sendCmd(CmdVO.UNREADY, G.me.id);
			ready.setText(R.string.ready);
			setBtnVisibility(false);
			break;
		case SHOW_READY:
			ready.setVisibility(Button.VISIBLE);
			ready.setText(R.string.ready);
			break;
		case DIALOG_INFO:
			if(clickPlayer!=null) {
				post(new Runnable() {
					public void run() {
						openInfo(clickPlayer);
						clickPlayer = null;
					}
				});
			}
			break;
		default :
			Bundle b = msg.getData();
			Player me = (Player)b.getSerializable("me");
			Player p1 = (Player)b.getSerializable("p1");
			Player p2 = (Player)b.getSerializable("p2");
			List<Map<String, Object>> list = new ArrayList<Map<String,Object>>();
			list.add(getMap(me));
			list.add(getMap(p1));
			list.add(getMap(p2));
			adapter = new SimpleAdapter(Game.this, list, R.layout.score_board_item, 
					new String[]{Player.NAME,Player.RESULT,Player.SCORE}, 
					new int[]{R.id.nick, R.id.result, R.id.score});
			openScoreBoard();
			if(levelUp) {
				ImageView view = new ImageView(this);
				view.setImageResource(R.drawable.levelup);
		        Toast toast = new Toast(this);
		        toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
		        toast.setDuration(Toast.LENGTH_SHORT);
		        toast.setView(view);
		        toast.show();
			}
			break;
		}
	}
	
	protected void deal(boolean success, CmdVO c) {
		super.deal(success, c);
		if(CmdVO.UPDATE.equals(c.cmd)) {
			GameView.netPlayers = J.fromJson(c.content, joinType);
			gv.loadNetPlayers();
		} else if(CmdVO.OUTLINE.equals(c.cmd)) { //有人断线
			GameView.netPlayers = J.fromJson(c.content, joinType);
			gv.outline = true;
			gv.loadNetPlayers();
		} else if(CmdVO.START.equals(c.cmd)) {
			GameView.netCards = J.fromJson(c.content, cardType);
			if(c.content==null) {
				G.me.distributor = true;
			} else {
				GameView.distributorId = c.playerId;
			}
			gv.beginNewGame();
		} else if(CmdVO.CALL.equals(c.cmd)) {
			if(G.curPlayer!=null && G.curPlayer.id.equals(c.playerId)) {
				int score = Integer.parseInt(c.content);
				if(score==0) {
					gv.notCall();
				} else {
					gv.callScore(score, G.curPlayer);
				} 
			}
		} else if(CmdVO.CARD.equals(c.cmd)) {
			if(G.curPlayer!=null && G.curPlayer.id.equals(c.playerId)) {
				G.curPlayer.willCards.clear();
				if(c.content!=null && !"null".equals(c.content)) {
					List<Card> cards = new ArrayList<Card>();
					List<Integer> indexs = J.fromJson(c.content, cardType2);
					for(Integer index : indexs) {
						cards.add(ImgMgr.cards[index]);
					}
					G.curPlayer.willCards.addAll(cards);
				}
				gv.cardMgr.commitCard(G.curPlayer);
			}
		} else if(CmdVO.READY.equals(c.cmd)) {
			for(Player p : gv.players) {
				if(c.playerId.equals(p.id)) p.ready = true;
			}
		} else if(CmdVO.UNREADY.equals(c.cmd)) {
			for(Player p : gv.players) {
				if(c.playerId.equals(p.id)) p.ready = false;
			}
		}
	}

	private void initWidgets() {
		more = (Button) findViewById(R.id.more);
		exit = (Button) findViewById(R.id.exit);
		backBoard = (Button) findViewById(R.id.back_board);
		ready = (Button) findViewById(R.id.ready);
	}
	
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if(keyCode==KeyEvent.KEYCODE_MENU){
			MusicService.playSound(Card.CLICK);
			if(!G.netMode) {
				gv.testCard=1;
				gv.beginNewGame();
				setBtnVisibility(false);
			}
			return true;
		} 
		if(keyCode==KeyEvent.KEYCODE_BACK) {
			MusicService.playSound(Card.CLICK);
			openExitDialog();
		}
		return super.onKeyDown(keyCode, event);
	}
	
	private void openInfo(Player p) {
		View view = View.inflate(this, R.layout.online_info, null);
		infoDialog = MyDialog.create(R.string.personal, view,  R.string.close);
		
		if(p!=null) { //如果已经有用户信息
			if(p.sex==null||p.sex==0) { //设置性别
				((RadioButton)view.findViewById(R.id.male)).setChecked(true);
			} else {
				((RadioButton)view.findViewById(R.id.female)).setChecked(true);
			}
			((ImageButton)view.findViewById(R.id.head)).setImageResource(Info.images[S.get(p.iconIndex)]);  //设置头像
			((TextView) view.findViewById(R.id.nick_tv)).setText(p.name); //设置姓名
			((TextView) view.findViewById(R.id.score)).setText(p.score==null?"0":p.score.toString());
			((TextView) view.findViewById(R.id.title)).setText(p.title==null?PlayerMgr.findTitle(0):p.title);
			((TextView) view.findViewById(R.id.win_rounds)).setText(p.win==null?"0":p.win.toString());
			((TextView) view.findViewById(R.id.lose_rounds)).setText(p.lose==null?"0":p.lose.toString());
			Integer num = Integer.parseInt(
					((TextView) view.findViewById(R.id.win_rounds)).getText().toString()) 
					+ Integer.parseInt(((TextView) view.findViewById(R.id.lose_rounds)).getText().toString());
			((TextView) view.findViewById(R.id.rounds)).setText(num.toString());
		}
	}
	
	protected Dialog onCreateDialog(int id) {
		switch(id) {
		case DIALOG_SUCCESS:
			if(successDialog==null) {
				successDialog = MyDialog.create(R.string.tip, R.string.task_success, R.string.sure);
			}
			return successDialog.dialog;
		case DIALOG_FAILED:
			if(failedDialog==null) {
				failedDialog = MyDialog.create(R.string.tip, R.string.task_failed, R.string.task_goon, R.string.exit);
			}
			return failedDialog.dialog;
		case DIALOG_OUT:
			if(outDialog==null) {
				outDialog = MyDialog.create(R.string.tip, R.string.score_not_enough, R.string.enter_normal, R.string.sure);
			}
			return outDialog.dialog;
		case DIALOG_FAILED_OUT:
			if(failedOutDialog==null) {
				failedOutDialog = MyDialog.create(R.string.tip, R.string.task_failed_out, R.string.enter_normal, R.string.sure);
			}
			return failedOutDialog.dialog;
		}
		return super.onCreateDialog(id);
	}
	
	private void openExitDialog() {
		int contentId = R.string.exit_normal_ask;
		if(G.gameMode==G.GAME_MODE_FUN) contentId = R.string.exit_fun_ask;
		if(G.gameMode==G.GAME_MODE_TASK) contentId = R.string.exit_task_ask;
		if(G.netMode)  {
			if(G.status == G.STATUS_ENTER || G.status==G.STATUS_BEGIN || G.status==G.STATUS_EXCITING) {
				contentId = R.string.exit_ing_ask;
			} else {
				contentId = R.string.exit_online_ask;
			}
		}
		exitDialog = MyDialog.create(R.string.tip, contentId, R.string.yes, R.string.no);
	}
	
	public void onClick(View v) {
		super.onClick(v);
		if(v==more || v==MyDialog.getBtn(failedDialog, R.string.task_goon)) { //"再来一盘" //"闯关失败" -->> "继续闯关"
			closeDialog();
			gv.beginNewGame();
			setBtnVisibility(false);
		} else if(v==exit) { //"退出游戏"
			openExitDialog();
		} else if(v==MyDialog.getBtn(exitDialog, R.string.yes) 
				|| v==MyDialog.getBtn(failedDialog, R.string.exit) 
				|| v==MyDialog.getBtn(successDialog)
				|| v==MyDialog.getBtn(outDialog, R.string.sure) 
				|| v==MyDialog.getBtn(failedOutDialog, R.string.sure)) { //回到PlayerInfo界面
			//点击了退出对话框的"是"  //"闯关失败" -->> "退出游戏" //闯关成功- ->>确定
			exitGame = true;
			closeDialog();
			G.startMusicService(MusicService.CMD_STOP);
			
			if(G.netMode) {
				post(new Runnable() { //发送离开桌子的信息
					public void run() {
						//如果强制退出,发送分数到服务器
						if(G.status == G.STATUS_ENTER || G.status==G.STATUS_BEGIN || G.status==G.STATUS_EXCITING) { 
							Map<Integer, Integer> scores = new HashMap<Integer, Integer>();
							scores.put(G.me.id, (gv.lastBottomScore==0?3:gv.lastBottomScore) * 2 * gv.scoreMultiple * -1);
							G.me.score = G.me.score - (gv.lastBottomScore==0?3:gv.lastBottomScore) * 2 * gv.scoreMultiple;
							for(Player p : gv.players) {
				    			if(p!=G.me) scores.put(p.id, (gv.lastBottomScore==0?3:gv.lastBottomScore) * 2* gv.scoreMultiple);
				        	}
							OnlineMgr.sendCmd(CmdVO.END, J.toJson(scores), G.me.id, null);
						} else {
							OnlineMgr.sendCmd(CmdVO.LEAVE_DESK);
						}
					}
				});
				
				G.me.ready = false;
				G.me.distributor = false;
			}
			
			finish();
			System.gc();
		} else if(v==MyDialog.getBtn(exitDialog, R.string.no)) {//点击了退出对话框的"否"
			MyDialog.dismiss(exitDialog);
		} else if(v==MyDialog.getBtn(outDialog, R.string.enter_normal) 
				|| v==MyDialog.getBtn(failedOutDialog, R.string.enter_normal)) { //从闯关模式转到普通模式-->>刷怪升级
			G.gameMode = G.GAME_MODE_NORMAL;
			closeDialog();
			setBtnVisibility(false);
			
			//重新设置电脑的分数  和 倍数
			gv.players.get(Player.LEFT_INDEX).score = Player.INIT_CPU_SCORE;
			gv.players.get(Player.RIGHT_INDEX).score = Player.INIT_CPU_SCORE;
			G.multiple = G.NORMAL_MULTIPLE; //2倍
			gv.beginNewGame();
		} else if(v==backBoard) {//"本局结算"
			closeDialog();
			setBtnVisibility(false);
			openScoreBoard();
		} else if(v==MyDialog.getBtn(taskTipDialog)) { //任务提示按钮
			MyDialog.dismiss(taskTipDialog);
		} else if(v == ready) {
			if(G.me.ready) { //如果之前的状态是准备,那么之前的按钮显示文字为"取消准备"
				//发送到服务器说取消准备
				OnlineMgr.sendCmd(CmdVO.UNREADY, G.me.id);
				G.me.ready = false;
				ready.setText(R.string.ready);
			} else {
				//发送到服务器说准备
				OnlineMgr.sendCmd(CmdVO.READY, G.me.id);
				G.me.ready = true;
				ready.setText(R.string.unready);
			}
		} else if (v==MyDialog.getBtn(infoDialog)) {
			MyDialog.dismiss(infoDialog);
		}
	}
	
	protected void onResume(){
		super.onResume();
	    G.startMusicService(MusicService.CMD_GO_ON);
	    //如果是EXCITING或BEGIN状态，进入游戏界面就不必改变状态值了,GameData.ENTER表示第一次进入游戏界面
	    if(G.status!=G.STATUS_EXCITING && G.status!=G.STATUS_BEGIN && G.status!=G.STATUS_ENTER) { 
	    	G.status = G.netMode?G.STATUS_LOADING:G.STATUS_ENTER;
	    	G.startMusicService(MusicService.TYPE_NORMAL, MusicService.CMD_PLAY);
	    }
	}

	protected void onStop() {
		super.onStop();
		if(G.status == G.STATUS_BEGIN||G.status == G.STATUS_ENTER||G.status == G.STATUS_EXCITING) {
			G.startMusicService(MusicService.CMD_PAUSE);
		}
	}
	
	private void openScoreBoard() {
		LayoutInflater mLayoutInflater = (LayoutInflater) getSystemService(LAYOUT_INFLATER_SERVICE);
		ViewGroup menuView = (ViewGroup) mLayoutInflater.inflate(R.layout.score_board, null, true);
		lv = (ListView) menuView.findViewById(R.id.scores);
		lv.setAdapter(adapter);
		
		scoreBoard = new PopupWindow(menuView, G.winWidth*2/3, G.winHeight*5/6, true);
		scoreBoard.setBackgroundDrawable(new BitmapDrawable());
		scoreBoard.setOnDismissListener(this);
		scoreBoard.setTouchInterceptor(this);
		scoreBoard.setAnimationStyle(R.style.Anim_ScoreBoard);
		scoreBoard.showAtLocation(findViewById(R.id.parent), Gravity.CENTER_HORIZONTAL | Gravity.TOP, 0, 0);
		scoreBoard.update();
	}
	
	private Map<String, Object> getMap(Player p) {
		Map<String, Object> map = null;
		if(p!=null) {
			map = new HashMap<String, Object>();
			map.put(Player.NAME, p.name);
			map.put(Player.RESULT, p.result);
			map.put(Player.SCORE, p.score);
		}
		return map;
	}
	
	public void onDismiss() {
		switch(what) {
		case TaskMgr.SUCCESS : //转到闯关界面
			if(G.me.taskId>TaskMgr.getTaskNums()) { //通过全部关卡
				SaveMgr saveMgr = new SaveMgr(this);
				if(saveMgr.get(SaveMgr.BEST_RECORD, 0)==0 || G.me.taskRounds < saveMgr.get(SaveMgr.BEST_RECORD, 0)) {
					saveMgr.update(SaveMgr.BEST_RECORD, G.me.taskRounds);
					saveMgr.commit();
				}
			}
			showDialog(DIALOG_SUCCESS);
			break;
		case TaskMgr.FAILED : //继续游戏  返回到闯关界面
			if(TaskMgr.out) {
				G.activityMap.get(Tasks.class).finish();
				showDialog(DIALOG_FAILED_OUT);
			} else {
				showDialog(DIALOG_FAILED);
			}
			break;
		case TaskMgr.ING :
			if(TaskMgr.out) {
				G.activityMap.get(Tasks.class).finish();
				showDialog(DIALOG_OUT);
			} else {
				setBtnVisibility(true);
			}
			break;
		default :
			setBtnVisibility(true);
		}
	}
	
	private void closeDialog() {
		MyDialog.dismiss(outDialog);
		MyDialog.dismiss(exitDialog);
		MyDialog.dismiss(successDialog);
		MyDialog.dismiss(failedDialog);
		MyDialog.dismiss(failedOutDialog);
		if(scoreBoard!=null) scoreBoard.dismiss();
	}

	public boolean onTouch(View v, MotionEvent event) {
		if(scoreBoard!=null) scoreBoard.dismiss();
		return false;
	}
	
	private void setBtnVisibility(boolean flag) {
		int i = flag?Button.VISIBLE:Button.GONE;
		if(G.netMode) {
			ready.setVisibility(i);
		} else {
			more.setVisibility(i);
			exit.setVisibility(i);
			backBoard.setVisibility(i);
		}
	}
}
