package rt.lanyo.game.ddz.logic;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import rt.lanyo.game.ddz.mgr.CardMgr;
import rt.lanyo.game.ddz.model.Card;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.view.GameView;
import android.util.Log;

public class BaseAI implements IAI {
	//private static String TAG="GameView";
    private CardMgr mgr;
	
    public  BaseAI(GameView view){
    	mgr= new CardMgr(view);
    }
    
    public List<UserCardStyle> findUserCardStyle(List<UserCardStyle>  ucs,CardStyle style){
    	List<UserCardStyle> list=new ArrayList<UserCardStyle>();
    	if(style==null) list.addAll(ucs);
    	else {
    	for(UserCardStyle uc:ucs){
    		if(style==uc.getStyle()){
    			list.add(uc);
    		}
    	}
    	}
    	return list;
    }
    /**
     * 从一堆牌中去除一张牌后，测试一下单牌数
     * @param list
     * @param c
     * @return
     */
    private int getSingleNum(List<Card> list,Card c){
    	List<Card> newList=new ArrayList<Card>(list);
    	newList.remove(c);
    	return findUserCardStyle(mgr.countUserCardStyle(newList),CardStyle.Single).size();
    }
    
    /**
     * 从一堆牌中去除一堆牌后，测试一下单牌数
     * @param list
     * @param clist
     * @return
     */
    private int getSingleNum(List<Card> list,List<Card> clist){
    	List<Card> newList=new ArrayList<Card>(list);
    	newList.removeAll(clist);
    	List<UserCardStyle> ucs= findUserCardStyle(mgr.countUserCardStyle(newList),CardStyle.Double);
    	for(UserCardStyle uc:ucs)newList.removeAll(uc.getCrads());
    	return findUserCardStyle(mgr.countUserCardStyle(newList),CardStyle.Single).size();
    }
    /**
     * 查找一张大于v的单牌，如何是任意单牌则v为0
     * @param list
     * @param ov
     * @param focusReturn 如果是focusReturn，则表示无论如何必须返回一个，需要拆散一些牌
     * @return
     */
    public Card findSingleCard(List<Card> list,int ov,boolean focusReturn){
    	List<UserCardStyle> allUcs=mgr.countUserCardStyle(list);
    	List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Single);
    	 //从单牌中查找
		for(UserCardStyle uc:ucs){
			int v=CardMgr.getRealValue(uc.getCrads().get(0));
			if(v>ov||v==Card.BIG_KING){
				//判断是否是大王，大王大于任何数
				return uc.getCrads().get(0);
				//break;
			}
		}
		Card card=null;
		if(focusReturn){
		List<Card> newList=new ArrayList<Card>();
		for(Card c:list){
			if(CardMgr.getRealValue(c)>ov){
				//Log.i("GameView", "ov:"+ov+",c:"+c.getValue());
				newList.add(c);
			}
		}
		int t=100;
		for(Card c:newList){
			int num=getSingleNum(newList, c);
			if(num<t){
				card=c;
				t=num;
			}
		}
		}
		return card;
    }
    
    /**
     * 查找最适合的对子
     * @param list
     * @param ov
     * @param focusReturn
     * @return
     */
    public List<Card> findDoubleCard(List<Card> list,int ov,boolean focusReturn){
    	List<UserCardStyle> allUcs=mgr.countUserCardStyle(list);
    	List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Double);
    	List<Card> cards=new ArrayList<Card>();
    	int t=100;
    	for(UserCardStyle uc:ucs){
    		List<Card> cl=uc.getCrads();
    		if(CardMgr.getRealValue(cl.get(0))>ov){
    			int n=getSingleNum(list,cl);
    			if(n<t){
    				t=n;
    				cards=cl;
    			}
    		}
    	}
    	return cards;
    }
    public List<Card> findMultiDoubleCard(List<Card> list,int ov,int num,boolean focusReturn){
    	List<UserCardStyle> allUcs=mgr.countUserCardStyle(list);
    	List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.MultiDouble);
    	List<Card> cards=new ArrayList<Card>();
    	
    	for(UserCardStyle uc:ucs){
    		List<Card> cl=uc.getCrads();
    		if(cl.size()<num)continue;
    		int j=-1;
    		for(int i=0;i<cl.size();i++ ){
    			if(CardMgr.getRealValue(cl.get(i))>ov){
    				j=i;
    				break;
    			}
    		}
    		if(j<0)continue;
    		cl=cl.subList(j, cl.size()-1);
    		if(cl.size()==num)return cl;
    		if(focusReturn){
    			//必须找一个
    		int t=100;	
    		for(int i=0;i<(cl.size()-num)/2;i++){
    			List<Card> clist=cl.subList(i*2, i*2+num);
    			int n=getSingleNum(list, clist);
    			if(n<t){
    				cards.clear();
    				cards.addAll(clist);
    				t=n;
    			}
    		}
    		}
    	}
    	return cards;
    }
    /**
     * 从一堆牌中找到一个大于ov，且张数为num的顺子
     * @param list 可选择的牌
     * @param ov 顺子最不的值
     * @param num 张数
     * @param focusReturn 是否必须返回一个
     * @return
     */
    public List<Card> findMultiSingleCard(List<Card> list,int ov,int num,boolean focusReturn){
    	List<UserCardStyle> allUcs=mgr.countUserCardStyle(list);
    	List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.MultiSingle);
    	
    	List<Card> cards=new ArrayList<Card>();
    	if(ucs.size()>0){
    		for(UserCardStyle uc:ucs){
    			//Log.i(TAG, "search....."+uc);
    			//截取最小的
    			int t=-1;
    			for(int i=0;i<uc.getCrads().size();i++){
    				if(CardMgr.getRealValue(uc.getCrads().get(i))>ov){
    					t=i;
    					break;
    				}
    			}
    			//Log.i(TAG, "search.....t:"+t);
    			if(t<0)continue;
    			List<Card> cl=uc.getCrads().subList(t, uc.getCrads().size()-1);
    			//Log.i(TAG, "search.....cl:"+cl);
    			//张数不够，返回
    			if(cl.size()<num)continue;
    			if(cl.size()==num)return cl;
    			//Log.i(TAG, "search.....");
    			if(focusReturn){//强制裁最好的牌
    			int snum=100;
    			for(int i=0;i<cl.size()-num;i++){
    				List<Card> ilist=cl.subList(i, i+num);
    				int sn=getSingleNum(list, ilist);
    				if(sn<snum){
    					cards.clear();
    					cards.addAll(ilist);
    					snum=sn;
    				}
    			}
    			}
    		}
    	}
    	
    	return cards;
    }
    
    public List<Card> findTribleCard(List<Card> list,int ov,boolean focusReturn){
    	List<UserCardStyle> allUcs=mgr.countUserCardStyle(list);
    	List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Triple);
    	List<Card> cards=new ArrayList<Card>();
    	int t=100;
    	for(UserCardStyle uc:ucs){
    		List<Card> cl=uc.getCrads();
    		if(CardMgr.getRealValue(cl.get(0))>ov){
    			int n=getSingleNum(list,cl);
    			if(n<t){
    				t=n;
    				cards=cl;
    			}
    		}
    	}
    	return cards;
    }
    //3带1
    public List<Card> findTribleOneCard(List<Card> list,int ov,boolean focusReturn){
    	List<Card> cards=findTribleCard(list,ov,focusReturn);
    	
    	if(!cards.isEmpty()){
    		List<Card> clist=new ArrayList<Card>(list);
    		clist.removeAll(cards);
    		Card card=findSingleCard(clist, 0, focusReturn);
    		if(card!=null){
    			cards.add(card);
    			return cards;
    		}
    		else cards.clear();
    	}
    	return cards;
    }
    //3带2
    public List<Card> findTribleTwoCard(List<Card> list,int ov,boolean focusReturn){
    	List<Card> cards=findTribleCard(list,ov,focusReturn);
    	if(!cards.isEmpty()){
    		List<Card> clist=new ArrayList<Card>(list);
    		clist.removeAll(cards);
    		List<Card> cs=findDoubleCard(clist, 0, focusReturn);
    		if(!cs.isEmpty()){
    			cards.addAll(cs);
    			return cards;
    		}
    		else cards.clear();
    	}
    	return cards;
    }
    //3顺飞机
    public List<Card> findMultiTribleCard(List<Card> list,int ov,int num,boolean focusReturn){
    	List<UserCardStyle> allUcs=mgr.countUserCardStyle(list);
    	List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.MultiTriple);
    	List<Card> cards=new ArrayList<Card>();
    	if(ucs.size()>0){
    		for(UserCardStyle uc:ucs){
    		List<Card> cl=uc.getCrads();
    		if(cl.size()<num)continue;
    		int j=-1;
    		for(int i=0;i<cl.size();i++ ){
    			if(CardMgr.getRealValue(cl.get(i))>ov){
    				j=i;
    				break;
    			}
    		}
    		if(j<0)continue;
    		if(cl.size()==num)return cl;
    		if(focusReturn){
    			//必须找一个
    		int t=100;	
    		cl=cl.subList(j, cl.size()-1);
    		for(int i=0;i<(cl.size()-num)/3;i++){
    			List<Card> clist=cl.subList(i*3, i*3+num);
    			int n=getSingleNum(list, clist);
    			if(n<t){
    				cards.clear();
    				cards.addAll(clist);
    				t=n;
    			}
    		}
    		}
    		}
    	}
    	return cards;
    }
    public List<Card> findBombCard(List<Card> list,int ov,boolean focusReturn){
    	List<UserCardStyle> allUcs=mgr.countUserCardStyle(list);
		List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Bomb);
		List<UserCardStyle> ucs2=findUserCardStyle(allUcs,CardStyle.DoubleKing);
		ucs.addAll(ucs2);
		List<Card> cards=new ArrayList<Card>();
		int t=100;
		for(int i=0;i<ucs.size();i++){
			if(CardMgr.getRealValue(ucs.get(i).getCrads().get(0))>ov){
			int n=getSingleNum(list, ucs.get(i).getCrads());
			if(n<t){
				t=n;
				cards=ucs.get(i).getCrads();
				System.out.println("new v:"+CardMgr.getRealValue(cards.get(0))+":"+ov);
			}
			}
		}
		
		return cards;
    }
    private void sortUserCardStyle(List<UserCardStyle>  list){
    	Collections.sort(list, new UserCardStyleComparator());
    }
    private int getCardNumbers(UserCardStyle uc){
    	CardStyle style=uc.getStyle();
    	if(style==CardStyle.Single||style==CardStyle.MultiSingle||style==CardStyle.Double||style==CardStyle.MultiDouble||style==CardStyle.Triple|| style==CardStyle.Bomb||style==CardStyle.DoubleKing||style==CardStyle.MultiTriple){
    		return uc.getCrads().size();
    	}
    	else if(style==CardStyle.TripleAndOne){
    		return 4;
    	}
    	else if(style==CardStyle.TripleAndTwo){
    		return 5;
    	}
    	else if(style==CardStyle.MultiTripleSwingSingle){
    		return uc.getCrads().size()+uc.getCrads().size()/3;
    	}
    	else if(style==CardStyle.MultiTripleSwingDouble){
    		return uc.getCrads().size()+uc.getCrads().size()/3*2;
    	}
    	else if(style==CardStyle.FourAndTwo){
    		return 6;
    	}
    	return 0;
    }
    
    
    private class UserCardStyleComparator implements Comparator<UserCardStyle>{
    	@Override
    	public int compare(UserCardStyle o1, UserCardStyle o2) {
    		int v1=getCardNumbers(o1);
    		int v2=getCardNumbers(o2);
    		if(o2.getStyle()==CardStyle.DoubleKing)return -1;
    		if(o2.getStyle()==CardStyle.Bomb){
    			if(o1.getStyle()==CardStyle.DoubleKing)return 1;
    			return -1;
    		}
    		int v=v2-v1;
    		return v;
    	}
       }
	@Override
	public List<Card> getBestCard(Player player, Player lastPlayer,
			CardStyle lastCardStyle, List<Card>[] lastRets) {
		return getBestCard(player, lastPlayer, lastCardStyle, lastRets, 0);
	}

	/**
	 * 检查用户出掉一张牌后胜利的机率
	 * @param myCards
	 * @param exceptsCard
	 * @return
	 */
	public int checkWillWinRate(List<Card> myCards,List<Card> exceptsCard){
		int ret=0;
		if(exceptsCard==null)exceptsCard=new ArrayList<Card>();
		//实现需要完善
		//如果除去exceptsCard后只有两手不重复的牌，则大于5，否则使用总牌型数减去小
		List<Card> cards=new ArrayList<Card>(myCards);
		cards.removeAll(exceptsCard);
		List<UserCardStyle> allUcs=mgr.countUserCardStyle(cards);
		if(allUcs.size()<2)ret=9;
		else {
			sortUserCardStyle(allUcs);
			//除掉一手牌余下牌的总张数
			int s=cards.size()-allUcs.get(0).getCrads().size();
			cards.removeAll(allUcs.get(0).getCrads());
			allUcs=mgr.countUserCardStyle(cards);
			//只有两手牌
			if(allUcs.size()<2)ret=7;
			else {
				//此处算法有问题，需要想一个公式来计算胜利的概率
				ret=-(s-(myCards.size()-exceptsCard.size()));
				if(ret>5)ret=5;
			}
		}
		return ret;
	}
	
	/**
	 * 先把炸弹提出来，不要出炸弹
	 * @param cards
	 * @return
	 */
	public List<Card> exceptsBombCard(List<Card> cards){
		List<UserCardStyle> allUcs=mgr.countUserCardStyle(cards);	
		List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Bomb);
		List<UserCardStyle> ucs2=findUserCardStyle(allUcs,CardStyle.DoubleKing);
		ucs.addAll(ucs2);
		List<Card> newCards=new ArrayList<Card>(cards);
		for(UserCardStyle uc:ucs){
			newCards.removeAll(uc.getCrads());
		}
		return newCards;
		
	}
	
	/**
	 * 根据计算出来的牌型来获得是否需要出牌
	 * @param player
	 * @param lastPlayer
	 * @param cards
	 * @param focuReturn
	 * @param newValue
	 * @param oldValue
	 * @return
	 */
	public boolean checkWinRateAndfocusReturn(Player player,Player lastPlayer,List<Card> cards,boolean focuReturn,int newValue,int oldValue){
		return this.checkWinRateAndfocusReturn(player, lastPlayer, cards, focuReturn, newValue, oldValue, 14, 2);
	}
	public boolean checkWinRateAndfocusReturn(Player player,Player lastPlayer,List<Card> cards,boolean focuReturn,int newValue,int oldValue,int maxValue,int differ){
		boolean ret=!cards.isEmpty();
		if(ret&&newValue>maxValue){
			//如果是对A或对2
			if(focuReturn){
				//进一步判断对方出的是什么牌，如果相差为1则可以出,比如对KK打对AA，如果对方打对AA，则根据情况出对22
				int dist=newValue-oldValue;
				if(dist>=differ){
					  //进一步判断是否需要
					  if(lastPlayer.cards.size()>2 && checkWillWinRate(player.cards,cards)<7){
						  //不用出
						  ret=false;
					  }
				}
			}
			else {
				//除非是自己的牌要完了，并且后面是牌是连牌，则可出
				//判断自己的牌是否要完了
				  if(checkWillWinRate(player.cards,cards)<7){
					  //不出
					  ret=false;
				  }
			}
			if(!ret && player.next.lord!=player.lord && player.next.cards.size()<3){
				//如果属于包牌,只有一张或者两张牌时，必须要
				ret=true;
			}
		}
		return ret;
	}
	private int getMaxCardValue(Player player){
		List<Card> allCards=new ArrayList<Card>();
		allCards.addAll(player.next.cards);
		allCards.addAll(player.next.cards);
		List<Card> lcs=CardMgr.arrangeCard(allCards)[0];
		if(lcs.size()>0){
			return CardMgr.getRealValue(lcs.get(lcs.size()-1));
		}
		return 14;
	}
	private void printCards(List<Card> cards){
		StringBuilder b=new StringBuilder("");
		for(Card c:cards)b.append(c.getValue()).append(",");
		Log.i("GameView","cards:"+b.toString());
	}
	
	@Override
	public List<Card> getTipCard(Player player, Player lastPlayer,
			CardStyle lastCardStyle, List<Card>[] lastRets, int num) {
		if(lastPlayer==null||lastPlayer==player){
			//如果单手是自己，则按getBestCard出
			return getBestCard(player, lastPlayer, lastCardStyle, lastRets, num);
		}
		//记录最后的结果
		List<Card> cards=new ArrayList<Card>();
		//把炸弹提取出来
		List<UserCardStyle> allUcs=mgr.countUserCardStyle(player.cards);
		List<UserCardStyle> bombUcs=findUserCardStyle(allUcs,CardStyle.Bomb);
		List<UserCardStyle> bombUcs2=findUserCardStyle(allUcs,CardStyle.DoubleKing);
		bombUcs.addAll(bombUcs2);
		List<Card> myCards=new ArrayList<Card>(player.cards);
		for(UserCardStyle uc:bombUcs){
			myCards.removeAll(uc.getCrads());
		}
		allUcs=mgr.countUserCardStyle(myCards);
		List<Card>[] rets=CardMgr.arrangeCard(myCards);
		List<UserCardStyle> choiceStyle=new ArrayList<UserCardStyle>();
		//如果最后一次出牌的人不是自己，表示必须出符合指定条件的牌
		int ov=CardMgr.getRealValue(lastRets[0].get(0));
		if(lastCardStyle==CardStyle.Single){
			 //如果是单牌
			   ov=CardMgr.getRealValue(lastRets[0].get(0));
			   List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Single);
		    	 //从单牌中查找
				for(UserCardStyle uc:ucs){
					int v=CardMgr.getRealValue(uc.getCrads().get(0));
					if(v>ov||v==Card.BIG_KING){
						//判断是否是大王，大王大于任何数
						choiceStyle.add(uc);
					}
				}
				if(rets.length>1&&!rets[1].isEmpty()){
					for(Card c:rets[1]){
						int v=CardMgr.getRealValue(c);
						if(v>ov){
							List<Card> cs=new ArrayList<Card>();
							cs.add(c);
							UserCardStyle uc=new UserCardStyle(CardStyle.Single, cs);
							choiceStyle.add(uc);
						}
					}
				}
				choiceStyle.addAll(bombUcs);
				if(choiceStyle.size()>num)cards.addAll(choiceStyle.get(num).getCrads());
		}
		else if(lastCardStyle==CardStyle.MultiSingle){
			//所有顺子
			List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.MultiSingle);
	    	if(ucs.size()>0){
	    		for(UserCardStyle uc:ucs){
	    			//截取最小的
	    			int t=-1;
	    			for(int i=0;i<uc.getCrads().size();i++){
	    				if(CardMgr.getRealValue(uc.getCrads().get(i))>ov){
	    					t=i;
	    					break;
	    				}
	    			}
	    			//Log.i(TAG, "search.....t:"+t);
	    			if(t<0)continue;
	    			List<Card> cl=uc.getCrads().subList(t, uc.getCrads().size()-1);
	    			
	    			//张数不够，返回
	    			if(cl.size()<num)continue;
	    			if(cl.size()==num){
	    				choiceStyle.add(new UserCardStyle(CardStyle.MultiSingle, cl));
	    				continue;
	    			}
	    			for(int i=0;i<cl.size()-num;i++){
	    				List<Card> ilist=cl.subList(i, i+num);
	    				choiceStyle.add(new UserCardStyle(CardStyle.MultiSingle, ilist));
	    			}
	    		}
	    	}
	    	choiceStyle.addAll(bombUcs);
			if(choiceStyle.size()>num)cards.addAll(choiceStyle.get(num).getCrads());
		}
		else if(lastCardStyle==CardStyle.Double){
			//所有对子
			List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Double);
			for(UserCardStyle uc:ucs){
	    		List<Card> cl=uc.getCrads();
	    		if(CardMgr.getRealValue(cl.get(0))>ov){
	    			choiceStyle.add(uc);
	    		}
			}
			choiceStyle.addAll(bombUcs);
			if(choiceStyle.size()>num)cards.addAll(choiceStyle.get(num).getCrads());
		}
		else if(lastCardStyle==CardStyle.MultiDouble){
			//所有连续对
			int cardNum=lastRets[0].size();
			List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.MultiDouble);
	    	for(UserCardStyle uc:ucs){
	    		List<Card> cl=uc.getCrads();
	    		if(cl.size()<cardNum)continue;
	    		int j=-1;
	    		for(int i=0;i<cl.size();i++ ){
	    			if(CardMgr.getRealValue(cl.get(i))>ov){
	    				j=i;
	    				break;
	    			}
	    		}
	    		if(j<0)continue;
	    		cl=cl.subList(j, cl.size()-1);
	    		if(cl.size()==cardNum){
	    			choiceStyle.add(new UserCardStyle(CardStyle.MultiDouble, cl));
    				continue;
	    		}
	    		for(int i=0;i<(cl.size()-cardNum)/2;i++){
	    			List<Card> clist=cl.subList(i*2, i*2+cardNum);
	    			choiceStyle.add(new UserCardStyle(CardStyle.MultiDouble, clist));
	    		}
	    	}
	    	choiceStyle.addAll(bombUcs);
			if(choiceStyle.size()>num)cards.addAll(choiceStyle.get(num).getCrads());
		}
		else if(lastCardStyle==CardStyle.Triple){
			ov=CardMgr.getRealValue(lastRets[2].get(0));
			List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Triple);
	    	for(UserCardStyle uc:ucs){
	    		List<Card> cl=uc.getCrads();
	    		if(CardMgr.getRealValue(cl.get(0))>ov){
	    			choiceStyle.add(uc);
	    		}
	    	}
	    	choiceStyle.addAll(bombUcs);
			if(choiceStyle.size()>num)cards.addAll(choiceStyle.get(num).getCrads());
		}
		else if(lastCardStyle==CardStyle.TripleAndOne){
			ov=CardMgr.getRealValue(lastRets[2].get(0));
			List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.TripleAndOne);
	    	for(UserCardStyle uc:ucs){
	    		List<Card> cl=uc.getCrads();
	    		if(CardMgr.getRealValue(cl.get(0))>ov){
	    			choiceStyle.add(uc);
	    		}
	    	}
	    	//找到带的牌
	    	List<UserCardStyle> singles=findUserCardStyle(allUcs,CardStyle.Single);
	    	List<Card> single=new ArrayList<Card>();
	    	if(singles.size()>0){
	    		single.addAll(singles.get(0).getCrads());
	    	}
	    	if(single.isEmpty())choiceStyle.clear();
	    	choiceStyle.addAll(bombUcs);
			if(choiceStyle.size()>num){
				cards.addAll(choiceStyle.get(num).getCrads());
				if(cards.size()<4)cards.addAll(single);
			}
		}
		else if(lastCardStyle==CardStyle.TripleAndTwo){
			ov=CardMgr.getRealValue(lastRets[2].get(0));
			List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.TripleAndTwo);
	    	for(UserCardStyle uc:ucs){
	    		List<Card> cl=uc.getCrads();
	    		if(CardMgr.getRealValue(cl.get(0))>ov){
	    			choiceStyle.add(uc);
	    		}
	    	}
	    	//找到带的牌
	    	List<UserCardStyle> singles=findUserCardStyle(allUcs,CardStyle.Double);
	    	List<Card> single=new ArrayList<Card>();
	    	if(singles.size()>0){
	    		single.addAll(singles.get(0).getCrads());
	    	}
	    	if(single.isEmpty())choiceStyle.clear();
	    	choiceStyle.addAll(bombUcs);
			if(choiceStyle.size()>num){
				cards.addAll(choiceStyle.get(num).getCrads());
				if(cards.size()<4)cards.addAll(single);
			}
		}
		else if(lastCardStyle==CardStyle.MultiTriple||lastCardStyle==CardStyle.MultiTripleSwingSingle||lastCardStyle==CardStyle.MultiTripleSwingDouble){
			ov=CardMgr.getRealValue(lastRets[2].get(0));
			int n=lastRets[2].size();
			int cardNum=n*3;
			//飞机带翅膀，还到所有的飞机
			List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.MultiTriple);
	    	if(ucs.size()>0){
	    		for(UserCardStyle uc:ucs){
	    		List<Card> cl=uc.getCrads();
	    		if(cl.size()<cardNum)continue;
	    		int j=-1;
	    		for(int i=0;i<cl.size();i++ ){
	    			if(CardMgr.getRealValue(cl.get(i))>ov){
	    				j=i;
	    				break;
	    			}
	    		}
	    		if(j<0)continue;
	    		if(cl.size()==cardNum){
	    			choiceStyle.add(new UserCardStyle(CardStyle.MultiTriple, cl));
    				continue;
	    		}
	    		
	    		cl=cl.subList(j, cl.size()-1);
	    		for(int i=0;i<(cl.size()-cardNum)/3;i++){
	    			List<Card> clist=cl.subList(i*3, i*3+cardNum);
	    			choiceStyle.add(new UserCardStyle(CardStyle.MultiTriple, clist));
	    		}
	    		}
	    	}
	    	
	    	//找到带的牌
	    	List<Card> single=new ArrayList<Card>();
			if(lastCardStyle==CardStyle.MultiTripleSwingSingle){
				List<Card> clist=new ArrayList<Card>(myCards);
				clist.removeAll(cards);
				for(int i=0;i<n;i++){
					Card card=findSingleCard(clist, 0, true);
		    		if(card!=null){
		    			single.add(card);
		    			clist.remove(card);
		    		}
		    		else single.clear();
				}
			}
			else if(lastCardStyle==CardStyle.MultiTripleSwingDouble){
				List<Card> clist=new ArrayList<Card>(myCards);
				clist.removeAll(cards);
				for(int i=0;i<n;i++){
					List<Card> cs=findDoubleCard(clist, 0, true);
		    		if(!cs.isEmpty()){
		    			single.addAll(cs);
		    			single.removeAll(cs);
		    		}
		    		else single.clear();
				}
			}
			
			if(single.isEmpty())choiceStyle.clear();
	    	choiceStyle.addAll(bombUcs);
			if(choiceStyle.size()>num){
				cards.addAll(choiceStyle.get(num).getCrads());
				if(cards.size()!=4)cards.addAll(single);
			}
			
			
		}
		else if(lastCardStyle==CardStyle.Bomb){
			//放炸弹
			ov=CardMgr.getRealValue(lastRets[3].get(0));
			for(UserCardStyle uc:bombUcs){
				if(CardMgr.getRealValue(uc.getCrads().get(0))>ov)
				choiceStyle.add(uc);
			}
			if(choiceStyle.size()>num)
			cards.addAll(choiceStyle.get(num).getCrads());
		}
		else if(lastCardStyle==CardStyle.FourAndTwo){//四带二
			for(UserCardStyle uc:bombUcs){
				if(CardMgr.getRealValue(uc.getCrads().get(0))>ov)
				choiceStyle.add(uc);
			}
			if(choiceStyle.size()>0&&player.cards.size()==6){
				//支持四带2
				cards.addAll(player.cards);
			}
			else {
				//全部是炸弹
				for(UserCardStyle uc:bombUcs){
					//只加小于4部分的炸弹
					if(CardMgr.getRealValue(uc.getCrads().get(0))<=ov)
					choiceStyle.add(uc);
				}
				if(choiceStyle.size()>num)cards.addAll(choiceStyle.get(num).getCrads());
			}
		}
		/*if(cards.isEmpty()){
			if((lastCardStyle!=CardStyle.Bomb&&lastCardStyle!=CardStyle.DoubleKing)){
				cards.addAll(findBombCard(player.cards,0,focuReturn));
			}
		}*/
		
		return cards;
	}

	@Override
	public List<Card> getBestCard(Player player, Player lastPlayer,
			CardStyle lastCardStyle, List<Card>[] lastRets, int num) {
		List<Card> cards=new ArrayList<Card>();
		int maxCardValue=getMaxCardValue(player);
		Log.i("GameView", "player:"+player.name+",lastPlayer:"+(lastPlayer!=null?lastPlayer.name:""));
		printCards(player.cards);
		boolean focuReturn=(lastPlayer!=null&&lastPlayer.lord)||player.lord;
		//把炸弹提取出来
		List<Card> myCards=exceptsBombCard(player.cards);
		if(lastPlayer!=null&&lastPlayer!=player){
			//如果最后一次出牌的人不是自己，表示必须出符合指定条件的牌
			int ov=CardMgr.getRealValue(lastRets[0].get(0));
			if(lastCardStyle==CardStyle.Single){
				 //如果是单牌
				   ov=CardMgr.getRealValue(lastRets[0].get(0));
				   //如果自己是地主或者要斗地主
				   Card card=findSingleCard(myCards, ov, focuReturn);
				   if(card==null && ov<15){
					  card=CardMgr.findCard(myCards, new Card(2,null,1)); //找一个2
					  if(card==null&&(ov<14))card=CardMgr.findCard(myCards, new Card(1,null,1));//找一个A
					  //如果是必须出牌
					  if(card==null&&focuReturn){
						  //如果没有找到A及2，进一步查找对子中最小的对子
						List<UserCardStyle> allUcs=mgr.countUserCardStyle(myCards);
						//把连对排除
					    List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.MultiDouble);
					    List<Card> tempCards=new ArrayList<Card>(myCards);
					    for(UserCardStyle us:ucs){
					    	tempCards.removeAll(us.getCrads());
					    }
					    //只从单对中找
					    allUcs=mgr.countUserCardStyle(tempCards);
					    ucs=findUserCardStyle(allUcs,CardStyle.Double);
					    for(UserCardStyle us:ucs){
					    	if(CardMgr.getRealValue(us.getCrads().get(0))>ov){
					    		card=us.getCrads().get(0);
					    		break;
					    	}
					    }
					  }
				   }
				   if(card!=null){
					cards.add(card);
				   if(!checkWinRateAndfocusReturn(player,lastPlayer,cards,focuReturn,CardMgr.getRealValue(card),ov,14,3)){
					   cards.clear();
				   }
				   }
			}
			else if(lastCardStyle==CardStyle.MultiSingle){
				//顺子
				cards.addAll(findMultiSingleCard(myCards, ov, lastRets[0].size(), focuReturn));
			}
			else if(lastCardStyle==CardStyle.Double){
				cards.addAll(findDoubleCard(myCards, ov, focuReturn));
				if(!cards.isEmpty()){
				if(!checkWinRateAndfocusReturn(player,lastPlayer,cards,focuReturn,CardMgr.getRealValue(cards.get(0)),ov)){
					   cards.clear();
				}
				}
			}
			else if(lastCardStyle==CardStyle.MultiDouble){
				cards.addAll(findMultiDoubleCard(myCards, ov,lastRets[0].size()*2, focuReturn));
			}
			else if(lastCardStyle==CardStyle.Triple){
				ov=CardMgr.getRealValue(lastRets[2].get(0));
				cards.addAll(findTribleCard(myCards, ov, focuReturn));
				if(!cards.isEmpty()){
					if(!checkWinRateAndfocusReturn(player,lastPlayer,cards,focuReturn,CardMgr.getRealValue(cards.get(0)),ov)){
						   cards.clear();
					}
					}
			}
			else if(lastCardStyle==CardStyle.TripleAndOne){
				ov=CardMgr.getRealValue(lastRets[2].get(0));
				cards.addAll(findTribleOneCard(myCards, ov, focuReturn));
				if(!cards.isEmpty()){
					if(!checkWinRateAndfocusReturn(player,lastPlayer,cards,focuReturn,CardMgr.getRealValue(cards.get(0)),ov)){
						   cards.clear();
					}
					}
			}
			else if(lastCardStyle==CardStyle.TripleAndTwo){
				ov=CardMgr.getRealValue(lastRets[2].get(0));
				cards.addAll(findTribleTwoCard(myCards, ov, focuReturn));
				if(!cards.isEmpty()){
					if(!checkWinRateAndfocusReturn(player,lastPlayer,cards,focuReturn,CardMgr.getRealValue(cards.get(0)),ov)){
						   cards.clear();
					}
					}
			}
			else if(lastCardStyle==CardStyle.MultiTriple||lastCardStyle==CardStyle.MultiTripleSwingSingle||lastCardStyle==CardStyle.MultiTripleSwingDouble){
				ov=CardMgr.getRealValue(lastRets[2].get(0));
				int n=lastRets[2].size();
				cards.addAll(findMultiTribleCard(myCards, ov, n*3, focuReturn));
				if(!cards.isEmpty()){
				if(lastCardStyle==CardStyle.MultiTripleSwingSingle){
					List<Card> clist=new ArrayList<Card>(myCards);
					clist.removeAll(cards);
					for(int i=0;i<n;i++){
						Card card=findSingleCard(clist, 0, true);
			    		if(card!=null){
			    			cards.add(card);
			    			clist.remove(card);
			    		}
			    		else cards.clear();
					}
				}
				else if(lastCardStyle==CardStyle.MultiTripleSwingDouble){
					List<Card> clist=new ArrayList<Card>(myCards);
					clist.removeAll(cards);
					for(int i=0;i<n;i++){
						List<Card> cs=findDoubleCard(clist, 0, true);
			    		if(!cs.isEmpty()){
			    			cards.addAll(cs);
			    			clist.removeAll(cs);
			    		}
			    		else cards.clear();
					}
				}
				}
				if(!cards.isEmpty()){
					if(!checkWinRateAndfocusReturn(player,lastPlayer,cards,focuReturn,CardMgr.getRealValue(cards.get(0)),ov,14,2)){
						   cards.clear();
					}
					}
			}
			else if(lastCardStyle==CardStyle.Bomb || (lastCardStyle==CardStyle.FourAndTwo && player.cards.size()!=6)){
				boolean out=focuReturn;
				if(!focuReturn){
					//看一下自己的是否要胜利了
					if(checkWillWinRate(player.cards,cards)>7){
						  //出
						out=true;
					}
				}
				if(out){
				//放炸弹
				ov=CardMgr.getRealValue(lastRets[3].get(0));
				cards.addAll(findBombCard(player.cards,ov,focuReturn));
				}
			}
			else if(lastCardStyle==CardStyle.FourAndTwo){//四带二
				List<UserCardStyle> allUcs=mgr.countUserCardStyle(player.cards);
				List<UserCardStyle> ucs=findUserCardStyle(allUcs,lastCardStyle);
				if(ucs.size()>0&&player.cards.size()==6){//一般4带二不要出，除非是直接关牌
					cards.addAll(player.cards);
				}
			}
			if(cards.isEmpty()){
				if(focuReturn){
				int winRate=checkWillWinRate(myCards,null);
				if((lastPlayer.cards.size()<2||winRate>5) && (lastCardStyle!=CardStyle.Bomb&&lastCardStyle!=CardStyle.DoubleKing)){
					//找一个炸弹来炸,前提是上一次出牌不是炸弹
					cards.addAll(findBombCard(player.cards,0,focuReturn));
				}
				}
				else {
					List<UserCardStyle> allUcs=mgr.countUserCardStyle(myCards);
					if(allUcs.size()<2){
						//找一个炸弹来炸
						cards.addAll(findBombCard(player.cards,(lastCardStyle!=CardStyle.Bomb&&lastCardStyle!=CardStyle.DoubleKing)?0:ov,focuReturn));
					}
				}
			}
		}
		else {
			//该用户出牌
			List<UserCardStyle> allUcs=mgr.countUserCardStyle(myCards);
			if(allUcs.size()<1){
				//只有一副牌并且有炸弹，则优先出炸弹
				cards.addAll(findBombCard(player.cards,0,true));
			}
			else {
			//可以优先出个数最多的小牌
			Log.i("GameView", "tipNum:"+num+",size:"+allUcs.size());
			sortUserCardStyle(allUcs);
			if(allUcs.size()>num){
				List<Card> choiceCard=null;
				for(int t=num;t<allUcs.size();t++){
				UserCardStyle uc= allUcs.get(t);
				//选中的牌旋转在choiceCard中
				choiceCard=new ArrayList<Card>(uc.getCrads());
				//Log.i(TAG, t+"choice");
				printCards(choiceCard);
				if(uc.getStyle()==CardStyle.Triple){
					List<Card> clist=new ArrayList<Card>(myCards);
		    		clist.removeAll(choiceCard);
		    		List<UserCardStyle> leftStyles=mgr.countUserCardStyle(clist);
	    			if(CardMgr.getRealValue(choiceCard.get(0))>13 && leftStyles.size()>1){
		    			continue;
		    		}
				}
				else if(uc.getStyle()==CardStyle.TripleAndOne){
					List<Card> clist=new ArrayList<Card>(myCards);
		    		clist.removeAll(choiceCard);
		    		Card card=findSingleCard(clist, 0, true);
		    		//ov=CardMgr.getRealValue(lastRets[3].get(0));
		    		if(card!=null){
		    			clist.remove(card);
		    			List<UserCardStyle> leftStyles=mgr.countUserCardStyle(clist);
		    			if(CardMgr.getRealValue(choiceCard.get(0))>13 && leftStyles.size()>1){
			    			continue;
			    		}
		    			int nv=CardMgr.getRealValue(card);
		    			if(nv<14 || leftStyles.size()<2){  //别把A,2及King带出去
		    				choiceCard.add(card);
		    				break;
		    			}
		    		}
		    		continue;//继承找下一幅牌
				}
				else if(uc.getStyle()==CardStyle.TripleAndTwo){
					List<Card> clist=new ArrayList<Card>(myCards);
		    		clist.removeAll(choiceCard);
		    		List<Card> cs=findDoubleCard(clist, 0, true);
		    		if(!cs.isEmpty()){
		    			clist.removeAll(cs);
		    			List<UserCardStyle> leftStyles=mgr.countUserCardStyle(clist);
		    			if(CardMgr.getRealValue(choiceCard.get(0))>13 && leftStyles.size()>1){
			    			continue;
			    		}
		    			int nv=CardMgr.getRealValue(cs.get(0));
		    			if(nv<14 || leftStyles.size()<2){  //别把A,2及King带出去
		    			choiceCard.addAll(cs);
		    			break;
		    			}
		    		}
		    		continue;
				}
				else if(uc.getStyle()==CardStyle.MultiTripleSwingSingle){
					int swingNum=uc.getCrads().size()/3;
					List<Card> clist=new ArrayList<Card>(myCards);
					clist.removeAll(choiceCard);
					for(int i=0;i<swingNum;i++){
						Card card=findSingleCard(clist, 0, true);
			    		if(card!=null){
			    			choiceCard.add(card);
			    			clist.remove(card);
			    		}
			    		else choiceCard.clear();
					}
					if(!choiceCard.isEmpty())break;
				}
				else if(uc.getStyle()==CardStyle.MultiTripleSwingDouble){
					int swingNum=uc.getCrads().size()/3;
					List<Card> clist=new ArrayList<Card>(myCards);
					clist.removeAll(choiceCard);
					for(int i=0;i<swingNum;i++){
						List<Card> cs=findDoubleCard(clist, 0, true);
			    		if(!cs.isEmpty()){
			    			choiceCard.addAll(cs);
			    			clist.removeAll(cs);
			    		}
			    		else choiceCard.clear();
					}
					if(!choiceCard.isEmpty())break;
				}
				else if(uc.getStyle()==CardStyle.FourAndTwo){
					List<Card> clist=new ArrayList<Card>(myCards);
		    		clist.removeAll(choiceCard);
		    		Card card=findSingleCard(clist, 0, true);
		    		if(card!=null){
		    			choiceCard.add(card);
		    			clist.remove(card);
		    		}
		    		card=findSingleCard(clist, 0, true);
			    	if(card!=null){
			    			choiceCard.add(card);
			    			break;
			    	}
			    	else continue;
			    		
				}
				else break;//其它牌直接跳出
				}
				
				if(choiceCard!=null&&!choiceCard.isEmpty())cards.addAll(choiceCard);
				//当为对子或者其它时，如果当牌大于K时，则尽量出单牌
				if(cards.size()>1){
					int v=CardMgr.getRealValue(cards.get(0));
					//最大牌-2
					if(v>maxCardValue-2){
						//则找单牌来出
						myCards.removeAll(cards);
						if(myCards.size()>0){
							//表示还有更多的牌
							allUcs=mgr.countUserCardStyle(myCards);
							//找一张单牌
							List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Single);
							if(ucs.size()>0){
								cards.clear();
								cards.addAll(ucs.get(0).getCrads());
							}
						}
					}
					else if(cards.size()==2){
						//找一张单牌,当是一对的时候，尽量先选小的单牌出
						List<UserCardStyle> ucs=findUserCardStyle(allUcs,CardStyle.Single);
						if(ucs.size()>0){
						int nv=CardMgr.getRealValue(ucs.get(0).getCrads().get(0));
						//当下一家只有一张时才尝试换成单牌
						if(player.next.cards.size()>1 && ucs.size()>0 && (v-nv>4 || (v>nv&&nv<6)) ){
							cards.clear();
							cards.addAll(ucs.get(0).getCrads());
						}
						}
					}
				}
			}
			}
			if(player.next.lord==player.lord&&player.next.cards.size()<2){
				//给用户出一张最小的牌
				if(player.cards.size()-cards.size()>player.next.cards.size()){//如果用户自己的牌出不完
				List<Card>[] rets=CardMgr.arrangeCard(myCards);
				cards.clear();
				cards.add(rets[0].get(0));
				}
			}
		}
		
		//如果下一家与我不是同一方
		if(!cards.isEmpty()){
			if(player.next.lord!=player.lord){
			//则近一步判断下一家的情况,如果只有一张牌，则选择大牌出
			int winRate=checkWillWinRate(player.next.cards,null);
			if(player.next.cards.size()<3||winRate>6){
				//斗地主
				List<Card>[] rets=CardMgr.arrangeCard(myCards);
				if(cards.size()==1){//出单牌
					cards.set(0, rets[0].get(rets[0].size()-1));
				}
				else if((cards.size()==2&&player.next.cards.size()==2 && cards.get(0).getValue()<Card.SMALL_KING)&&rets[1].size()>1){
					//出一对,排除一对王
					cards.clear();
					cards.add(rets[1].get(rets[1].size()-1));
					cards.add(CardMgr.findCard(rets[0], cards.get(0)));
				}
			}
			}
			if(lastPlayer!=null && player!=lastPlayer&& player.lord==lastPlayer.lord){
				if((lastPlayer.lastCards.size()<3)){
					//出对子或单个的时候
					int v=CardMgr.getRealValue(lastPlayer.lastCards.get(0));
					if(v>13){
						int winRate=checkWillWinRate(player.cards,cards);
						//近一步判断是否可以出完余下的牌，否则不出
						if(winRate<6){
							//可以不用出
							cards.clear();
						}
					}
				}
			}
		}
		printCards(cards);
		return cards;
	}

}

