package rt.lanyo.game.ddz.mgr;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import rt.lanyo.game.ddz.R;
import rt.lanyo.game.ddz.logic.CardStyle;
import rt.lanyo.game.ddz.logic.UserCardStyle;
import rt.lanyo.game.ddz.model.Card;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.service.MusicService;
import rt.lanyo.game.ddz.util.G;
import rt.lanyo.game.ddz.util.J;
import rt.lanyo.game.ddz.view.GameView;
import rt.lanyo.game.ddz.vo.CmdVO;
/**
 * 管理牌
 */
@SuppressWarnings("unchecked")
public class CardMgr {
	private static GameView gv;
	public CardMgr(GameView view) {
		gv = view;
	}
	
	private static boolean containCard(List<Card> list,Card card){
		return findCard(list,card)!=null;
	}
	
	public static Card findCard(List<Card> list, Card card) {
		for (Card c : list) {
			if (c.getValue() == card.getValue())
				return c;
			else if ((c.getValue() == Card.BIG_KING && card.getValue() == Card.SMALL_KING)
					|| (c.getValue() == Card.SMALL_KING && card.getValue() == Card.BIG_KING))
				return c;
		}
		return null;
	}
	
	public void sortCards(List<Card> list){
		Collections.sort(list,new CardComparator() );
	}
	
	public static List<Card>[] arrangeCard(List<Card> list){
		Collections.sort(list,new CardComparator() );
		List<Card>[] rets=new ArrayList[4];
		rets[0]=new ArrayList<Card>();
		rets[1]=new ArrayList<Card>();
		rets[2]=new ArrayList<Card>();
		rets[3]=new ArrayList<Card>();
		for(Card c:list){
			if(!containCard(rets[0],c)){
				rets[0].add(c);
			}
			else if(!containCard(rets[1],c)){
				rets[1].add(c);
			}
			else if(!containCard(rets[2],c)){
				rets[2].add(c);
			}
			else if(!containCard(rets[3],c)){
				rets[3].add(c);
			}
		}
		return rets;
	}
	
	public static int getRealValue(Card c){
		int v=c.getValue();
		if(v==1)return 14;
		else if(v==2)return 15;
		else return v;
	}
	
	public List<UserCardStyle> countUserCardStyle(List<Card> list){
		List<UserCardStyle> ucslist = new ArrayList<UserCardStyle>();
		if (list.size() == 1){
			ucslist.add(new UserCardStyle(CardStyle.Single, new ArrayList<Card>(list)));
		} else if(list.size() == 2){
			int one = list.get(0).getValue();
			int two = list.get(1).getValue();
			if(one==two){
				ucslist.add(new UserCardStyle(CardStyle.Double, new ArrayList<Card>(list)));
			}
			
			//王炸
			if(one+two == Card.BIG_KING+Card.SMALL_KING) {
				ucslist.add(new UserCardStyle(CardStyle.DoubleKing, new ArrayList<Card>(list)));
			}
		} else {
			//获得一个用户的所有牌型
			List<Card>[] rets = arrangeCard(list);
			//从第一排检查顺子情况
			if(rets[0].size()>=5){
				List<Card> sl=new ArrayList<Card>();
				sl.add(rets[0].get(0));
				for(int i=1;i<rets[0].size();i++){
					int v=getRealValue(rets[0].get(i));
					if(v<15&&v-getRealValue(rets[0].get(i-1))==1){
						sl.add(rets[0].get(i));
					} else {
						//有断牌
						if(sl.size()>4){//有5张及以上单独连续则表示有连子
							//查看前面的牌是否已经形成顺子，如果是则添加到顺子中
							ucslist.add(new UserCardStyle(CardStyle.MultiSingle,new ArrayList<Card>(sl)));
						}
						sl.clear();
						sl.add(rets[0].get(i));
					}
				}
				if(sl.size()>4){//有5张及以上单独连续则表示有连子
					//查看前面的牌是否已经形成顺子，如果是则添加到顺子中
					ucslist.add(new UserCardStyle(CardStyle.MultiSingle,new ArrayList<Card>(sl)));
				}
			}
			
			//从第二排检查对子情况
			if (rets[1].size() > 0) {
				// 连续的
				List<Card> sl = new ArrayList<Card>();
				// 对子
				for (int i = 0; i < rets[1].size(); i++) {
					Card card = rets[1].get(i);
					int v = getRealValue(card);
					List<Card> dl = new ArrayList<Card>();
					dl.add(card);
					if(card.getValue()==Card.BIG_KING) {
						dl.add(findCard(rets[0], new Card(Card.SMALL_KING, null, 1)));
					} else if((card.getValue()==Card.SMALL_KING)) {
						dl.add(findCard(rets[0], new Card(Card.BIG_KING, null, 2)));
					} else {
						dl.add(findCard(rets[0], card));
					}
					
					ucslist.add(new UserCardStyle(
							(card.getValue() == Card.BIG_KING || card.getValue() == Card.SMALL_KING) ? CardStyle.DoubleKing
									: CardStyle.Double, new ArrayList<Card>(dl)));
					// 把这一对添加到连续的对子中
					if (i > 0) {
						// 处理顺子对
						if (v < Card.TWO && v - getRealValue(rets[1].get(i - 1)) == 1) {
							sl.addAll(dl);
						} else {
							// 有断牌
							if (sl.size() > 5) {//有6张牌
								// 查看前面的牌是否已经形成顺子，如果是则添加到顺子中
								ucslist.add(new UserCardStyle(CardStyle.MultiDouble,new ArrayList<Card>(sl)));
							}
							sl.clear();
							sl.addAll(dl);
						}
					} else {// 第一次直接加入
						sl.addAll(dl);
					}
				}
				if (sl.size() > 5) {//有6张牌
					//查看前面的牌是否已经形成顺子，如果是则添加到顺子中
					ucslist.add(new UserCardStyle(CardStyle.MultiDouble,new ArrayList<Card>(sl)));
				}
			}
						
			//从第三排检查三带一，三带二及连带的情况
			if(rets[2].size()>0){
				List<Card> sl = new ArrayList<Card>();
				for(int i=0;i<rets[2].size();i++){
					int n=i+1;
					Card card = rets[2].get(i);
					int v = getRealValue(card);
					List<Card> dl = new ArrayList<Card>();
					dl.add(card);
					dl.add(findCard(rets[0], card));
					dl.add(findCard(rets[1], card));
					ucslist.add(new UserCardStyle(CardStyle.Triple,new ArrayList<Card>(dl)));
					if(rets[0].size()>1){
						//可以三带一
						//具体带的牌到真正出的时候再确定
						ucslist.add(new UserCardStyle(CardStyle.TripleAndOne,new ArrayList<Card>(dl)));
					}
					if(rets[1].size()>1){
						//可以三带二
						//具体带的牌到真正出的时候再确定
						ucslist.add(new UserCardStyle(CardStyle.TripleAndTwo,new ArrayList<Card>(dl)));
					}
					if(i>0){
						if (v < Card.TWO && v - getRealValue(rets[2].get(i - 1)) == 1) {
							//System.out.println(v);
							sl.addAll(dl);
							//每次都加一个新的3顺
							if (sl.size() > 5) {//有6张牌
								n=sl.size()/3;
								// 三顺带
								ucslist.add(new UserCardStyle(CardStyle.MultiTriple,new ArrayList<Card>(sl)));
								//判断是否可以带单牌
								if(rets[0].size()-n>=n){
									ucslist.add(new UserCardStyle(CardStyle.MultiTripleSwingSingle,new ArrayList<Card>(sl)));
								}
								//System.out.println("swing-double:"+(rets[1].size()-n)+":"+n+":"+rets[1].size());
								//判断是否可以带对子
								if(rets[1].size()-n>=n){
									ucslist.add(new UserCardStyle(CardStyle.MultiTripleSwingDouble,new ArrayList<Card>(sl)));
								}
							}
						} else {
							sl.clear();
							sl.addAll(dl);
						}
					}
					else {
						sl.addAll(dl);
					}
				}
			}
			//从第四排检查炸弹情况
			if(rets[3].size()>0){
				for(int i=0;i<rets[3].size();i++){
					Card card = rets[3].get(i);
					List<Card> dl = new ArrayList<Card>();
					dl.add(card);
					dl.add(findCard(rets[0], card));
					dl.add(findCard(rets[1], card));
					dl.add(findCard(rets[2], card));
					//添加炸弹牌型
					ucslist.add(new UserCardStyle(CardStyle.Bomb,new ArrayList<Card>(dl)));
					if(list.size()>5){
						//可以4带二
						ucslist.add(new UserCardStyle(CardStyle.FourAndTwo,new ArrayList<Card>(dl)));
					}
				}
			}
			//处理单牌
		}
		List<Card> slist=new ArrayList<Card>(list);
		List<Card> dblist=new ArrayList<Card>(list);
		List<Card> multiSingles=new  ArrayList<Card>();
		for(UserCardStyle ucs:ucslist){
			int ov=getRealValue(ucs.getCrads().get(0));
			if(ucs.getStyle()==CardStyle.MultiSingle){
				multiSingles.addAll(ucs.getCrads());
			}
			if(ucs.getStyle()==CardStyle.Double){
				if(ov>13){
					//对A,对2可以撤为单牌出,则不需要从slist中删除
					continue;
				}
				dblist.addAll(ucs.getCrads());
			}
			else if(ucs.getStyle()==CardStyle.Triple||ucs.getStyle()==CardStyle.TripleAndOne||ucs.getStyle()==CardStyle.TripleAndTwo){
				//3对A及3对2当成单牌打
				if(ov>13)continue;
			}
			slist.removeAll(ucs.getCrads());
		}
		//如果multiSingle不为空，则除去后看单牌
		if(!multiSingles.isEmpty()){
			List<Card> clist=new ArrayList<Card>(list);
			clist.removeAll(multiSingles);
			List<Card>[] rets = arrangeCard(clist);
			if(rets.length>0&&!rets[0].isEmpty()){
				clist=rets[0];
				if(rets.length>1&&!rets[1].isEmpty()){
					for(Card c:rets[1]){//删除仍然是一对的牌
						clist.remove(findCard(clist, c));
					}
				}
			}
			for(Card c:clist){
				if(!slist.contains(c))slist.add(c);
			}
		}
		if(slist.size()>0){
			sortCards(slist);//对单牌进行排序
			for(Card c:dblist){
				if(!slist.contains(c))slist.add(c);
			}
			for(Card c:slist){
				List<Card> dl = new ArrayList<Card>();
				dl.add(c);
				ucslist.add(new UserCardStyle(CardStyle.Single,new ArrayList<Card>(dl)));
			}
		}
		return ucslist;
	}
	
	
	/**
	 * 给几张牌，输出牌型
	 * @param list
	 * @return
	 */
	public static CardStyle getCardStyle(List<Card> list) {
		if(list.size() == 1) return CardStyle.Single;
		if(list.size()==2){
			int one = list.get(0).getValue();
			int two = list.get(1).getValue();
			if(one == two){
				return CardStyle.Double;
			} else if(one+two == Card.BIG_KING+Card.SMALL_KING) {
				return CardStyle.DoubleKing;
			} else return null;//非法组合
		}
		List<Card>[] rets = arrangeCard(list);
		if (rets[0].size() >= 5 && rets[0].size() == list.size()) {//说明全部是单牌
			// 检查顺子
			int begin = getRealValue(rets[0].get(0));
			int end = getRealValue(rets[0].get(rets[0].size() - 1));
			// System.out.println("b:"+begin+",e:"+end+",s:"+(end-begin));
			// 最小为L
			if (begin < 11 && end < 15 && end - begin == list.size() - 1) {
				return CardStyle.MultiSingle;
			}
			return null;//非法组合
		}
		// 判断炸弹或者4带2或者4带两对
		else if (rets[3].size() > 0 && (list.size() == 6 || list.size() == 4 )) {// 第四个数组中有值，说明有四个相同数值的牌
			if ((rets[0].size() == 3 || rets[1].size() == 2)
					&& rets[2].size() == 1)  //用来区别四带还是四带2对
				return CardStyle.FourAndTwo;// 四带二
			else if (rets[0].size() == 1)
				return CardStyle.Bomb;// 炸弹
			else return null;
		} else if (rets[2].size() > 0) {//有三条且不是4条
			// 有三个
			// 是否三顺
			if (list.size() == 3) return CardStyle.Triple;
			// 三带一
			if (list.size() == 4) return CardStyle.TripleAndOne;
			// 三带2，可能还有问题
			if (list.size() == 5 && rets[0].size() == 2 && rets[1].size() == 2)
				return CardStyle.TripleAndTwo;// 三带二
			// 检查是否飞机
			if (rets[2].size() > 1) { //至少有两个三条
				int begin = getRealValue(rets[2].get(0));
				int end = getRealValue(rets[2].get(rets[2].size() - 1));
				// N顺，可能还有问题，需要验证,刚好等于3*n张牌
				int n = rets[2].size();// n表示n顺
				if (end < 15 && end - begin == n - 1) {
					// 成功为n顺	
					if (list.size() / 3 == n&&list.size()%3==0) {
						return CardStyle.MultiTriple;
					} else {
						// 是否为带单个的翅膀，余下的牌刚好为n张
						if (list.size() - n * 3 == n){
							return CardStyle.MultiTripleSwingSingle;
						}
						else if (list.size() - n * 3 == n * 2) {
							// 进一步检查是否余下的牌全为对子,如果是对子，则全部均应该在1，2两栏上
							if (rets[0].size() == rets[1].size()
									&& rets[1].size() == n * 2)
								return CardStyle.MultiTripleSwingDouble;
						}
					}
				} else return null;//连不成飞机的三条
			}
		}
		else if(rets[1].size()>2 && rets[1].size()==rets[0].size()){
			//连对
			int begin = getRealValue(rets[1].get(0));
			int end = getRealValue(rets[1].get(rets[1].size() - 1));
			if (end < 15 && end - begin == rets[1].size()- 1) {
				return CardStyle.MultiDouble;
			}
		}
		return null;
	}
   
   private static class CardComparator implements Comparator<Card>{
		@Override
		public int compare(Card o1, Card o2) {
			int v1=getRealValue(o1);
			int v2=getRealValue(o2);
			int v=v1-v2;
			if(v==0)v=o1.getColor()-o2.getColor();
			return v;
		}
   }
   
   /**
	 * 分牌给用户
	 * @param players
	 * @return
	 */
	public List<Card> distributeCard(List<Player> players){
		for(Player p:players)p.cleanCard();
		List<Card> cards = cleanCards();
		for(int i=0;i<17;i++){
			players.get(Player.LEFT_INDEX).cards.add(cards.get(i*3));
			players.get(Player.RIGHT_INDEX).cards.add(cards.get(i*3+1));
			players.get(Player.ME_INDEX).cards.add(cards.get(i*3+2));
		}
		
		for(Player p:players)sortCards(p.cards);
		List<Card> bottom=new ArrayList<Card>();
		bottom.add(cards.get(51));
		bottom.add(cards.get(52));
		bottom.add(cards.get(53));
		sortCards(bottom);
		return bottom;
	}
	
	/**
	 * 洗牌
	 * @return
	 */
	public List<Card> cleanCards(){
		if(ImgMgr.cards==null) ImgMgr.cards = new Card[54];
		List<Card> cs=new ArrayList<Card>(Arrays.asList(ImgMgr.cards));
		if(gv.testCard==1){
			gv.testCard=0;
		}
		else {  
		for(int i=0;i<100;i++){
		int a=	(int)(Math.random()*ImgMgr.cards.length);
		int b=	(int)(Math.random()*ImgMgr.cards.length);
		Card c=cs.get(a);
		cs.set(a, cs.get(b));
		cs.set(b, c);
		}
		}
		return cs;
	}
	
	public List<Card> distributeCard2(List<Player> players){
		for(Player p:players)p.cleanCard();
		for(int i=0;i<17;i++){
			players.get(Player.LEFT_INDEX).cards.add(ImgMgr.cards[i]);
		}
		for(int i=17;i<34;i++){
			players.get(Player.RIGHT_INDEX).cards.add(ImgMgr.cards[i]);
		}
		for(int i=34;i<51;i++){
			players.get(Player.ME_INDEX).cards.add(ImgMgr.cards[i]);
		}
		for(Player p:players)sortCards(p.cards);
		List<Card> bottom=new ArrayList<Card>();
		bottom.add(ImgMgr.cards[51]);
		bottom.add(ImgMgr.cards[52]);
		bottom.add(ImgMgr.cards[53]);
		sortCards(bottom);
		return bottom;
	}
	
	//娱乐模式的发牌
	public List<Card> distributeCard3(List<Player> players) {
		for(Player p:players)p.cleanCard();
		List<Card> cs=new ArrayList<Card>(Arrays.asList(ImgMgr.cards));
		List<Card> bottom = new ArrayList<Card>();
		int ran = (int)(Math.random()*13) + 1;
		List<Card> removeList = new ArrayList<Card>();
		List<Integer> temp= new ArrayList<Integer>();
		int begingNO = (int)(Math.random()*3);
		for(int i=0; i<12; i++){
			if(!temp.contains(ran)){
				temp.add(ran);
				if(ran!=13){
					for(int j=ran*4;j<ran*4+4;j++){
						players.get(begingNO%3).cards.add(ImgMgr.cards[j]);
						removeList.add(ImgMgr.cards[j]);
					}
				} else if(ran == 13){
					players.get(begingNO%3).cards.add(ImgMgr.cards[52]);
					players.get(begingNO%3).cards.add(ImgMgr.cards[53]);
					removeList.add(ImgMgr.cards[53]);
					removeList.add(ImgMgr.cards[52]);
				}
			}
			begingNO++;
			ran = (int)(Math.random()*13) + 1;
		}
		cs.removeAll(removeList);
		//洗剩下的牌
		for(int i=0;i<100;i++){
			int a=	(int)(Math.random()*cs.size());
			int b=	(int)(Math.random()*cs.size());
			Card c=cs.get(a);
			cs.set(a, cs.get(b));
			cs.set(b, c);
		}
		//将剩下的牌发出去
		int j=0;
		for(int i=0; i<3; i++){
			while(players.get(i).cards.size() < 17 && j<cs.size()){
				players.get(i).cards.add(cs.get(j++));
			}
		}
		bottom.add(cs.get(cs.size()-1));
		bottom.add(cs.get(cs.size()-2));
		bottom.add(cs.get(cs.size()-3));
		sortCards(bottom);
		for(Player p:players)sortCards(p.cards);
		return bottom;
	}

	public void getPlayerCard(Player player, List<Card> curCard) {
		if (!gv.netMode) {
			List<Card> list = gv.playerAI.getBestCard(player, G.lastPlayer,
					gv.lastCardStyle, gv.lastRets);
			synchronized (player.willCards) {
				player.willCards.addAll(list);
			}
		}
	}
	
	//出牌
	public void commitCard(final Player player){
		if(G.status == G.STATUS_END) return;
		
		ImgMgr.stopClock(); //让时钟消失
		
		gv.tipNum=0;
		if(player!=G.me){
			//获得用户要出的牌，并自动存储到willCards中
			if(!gv.netMode) getPlayerCard(player,null);
		} else {
			BtnMgr.resetHandleBtns();
		}
		
		//提交出牌内容
		if(gv.netMode && player==G.me){
			List<Integer> indexs = new ArrayList<Integer>();
			for(Card c : player.willCards) {
				indexs.add(c.getIndex());
			}
			OnlineMgr.sendCmd(CmdVO.CARD, J.toJson(indexs), player.id, null);
		}
		
		//提交牌
		if(!player.willCards.isEmpty()){
			if(player.lord) gv.lordCount++;  //如果地主出牌了
			
			//记录当前出的牌
			gv.lastCardStyle = getCardStyle(player.willCards);
			gv.lastRets = arrangeCard(player.willCards);
			
			//用于音效判断
			MusicService.cur = player;
			MusicService.style = gv.lastCardStyle;
			MusicService.playCardSound();
			MusicService.lastPlayer = player;
			MusicService.lastStyle= gv.lastCardStyle;
			
			G.lastPlayer = player;
			if(isBomb(gv.lastCardStyle)){
				if(ImgMgr.needAnim) ImgMgr.bombed = true;//用于画炸弹图片
				gv.scoreMultiple*=G.multiple;
			}
			if(isPlane(gv.lastCardStyle)){
				if(ImgMgr.needAnim) ImgMgr.planed = true;
			}
			
		    player.commitCards();
		    if(player.cards.isEmpty()){
		    	boolean spring = true;//对春天的判断
		    	if(player.lord){ //如果winner是地主
		    		for(Player p : gv.players) {  
			    		if(p!=player) {
				    		if(p.cards.size()<17) spring = false;
			    		}
			    	}
		    	} else { //如果winner不是地主
	    			if(gv.lordCount>1) spring = false;
	    		}
		    	if(spring) gv.scoreMultiple*=G.multiple;
		    	//胜利........
		    	gv.showWinner(player);
		    	return;
		    }
		    
		    //音乐放到后边
		    if(isBomb(gv.lastCardStyle) && G.status!=G.STATUS_END){
				if(!MusicService.exciting) {
					G.status = G.STATUS_EXCITING;
					G.startMusicService(MusicService.TYPE_EXCITING, MusicService.CMD_PLAY);
					MusicService.exciting = true;
				}
			}
		} else {
			//不要
			MusicService.playSound(player.sex, (int)(Math.random()*4) + Card.BU_YAO1);
			MsgMgr.append(player, R.string.bu_yao);
		}
		
		//轮到下个玩家出牌
		gv.handler.postDelayed(new Runnable() {
			public void run() {
				if((G.status==G.STATUS_BEGIN||G.status==G.STATUS_EXCITING)&&G.curPlayer!=null){
						G.curPlayer=player.next;
						ImgMgr.beginClock(); //显示时钟
						synchronized (G.curPlayer.lastCards) {
							G.curPlayer.lastCards.clear();
						}
						if(G.curPlayer==G.me){
							gv.tipNum = 0;
							gv.tipCards = gv.playerAI.getTipCard(G.me, G.lastPlayer, gv.lastCardStyle, gv.lastRets, gv.tipNum);
							if(gv.tipCards.isEmpty()) G.toast(R.string.no_cards);
							BtnMgr.showHandleBtns();
						} else {
							//如果是网络模式，继续提交，否则停止
							if(!gv.netMode) commitCard(G.curPlayer);
						}
				}
			}
		}, ImgMgr.planed||ImgMgr.bombed?G.SPECIAL_ANIM_TIME:G.PLAYER_STEP_TIME);
	}
	
	public boolean isBomb(CardStyle style) {
		if(style==CardStyle.Bomb||style==CardStyle.DoubleKing) return true;
		return false;
	}
	
	public boolean isPlane(CardStyle style) {
		if(style==CardStyle.MultiTriple||style==CardStyle.MultiTripleSwingSingle||style==CardStyle.MultiTripleSwingDouble) return true;
		return false;
	}
}
