package rt.lanyo.game.ddz.mgr;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import rt.lanyo.game.ddz.Base;
import rt.lanyo.game.ddz.Game;
import rt.lanyo.game.ddz.R;
import rt.lanyo.game.ddz.Room;
import rt.lanyo.game.ddz.model.Card;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.util.G;
import rt.lanyo.game.ddz.util.J;
import rt.lanyo.game.ddz.view.GameView;
import rt.lanyo.game.ddz.vo.CmdVO;
import android.app.Activity;
import com.google.gson.Gson;

/**
 * 在线网络连接管理
 * 负责把牌提交到网络，以及获取牌的信息等
 * @author 蔡世友
 *
 */
public class OnlineMgr implements Runnable{
	
	private static final String IP = G.context.getString(R.string.ip);
	private static final String PORT = G.context.getString(R.string.port);
	private static final byte CMD_OVER_TIME = 20; //客户端发送命令等待超过10S就代表请求失败
	public static final int Time_OUT = 70000;//超时时间
	private Socket socket;
	private long cmdStart; //客户端发送一个请求的开始时间
	private InputStream in;
	private OutputStream out;
	private static OnlineMgr singleton;
	private Map<String,Callback> callbacks=new HashMap<String,Callback>();
	private Thread reciveMsgThread;
	private Thread heartThread;
	private boolean stop;
	private Gson gson;
	
	private OnlineMgr(){
		gson = new Gson();
		reciveMsgThread=new Thread(this);
		reciveMsgThread.start();
		heartThread = new Thread(new Runnable() {
			public void run() {
				while(true) {
					if(G.status!=G.STATUS_ENTER && G.status!= G.STATUS_BEGIN && G.status!=G.STATUS_EXCITING) {
						try {
							OnlineMgr.sendCmd(CmdVO.HEARTBEAT);
							Thread.sleep(50000);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			} 
		});
		heartThread.start();
	}
	public static OnlineMgr getInstance(){
		if(singleton==null){
			singleton=new OnlineMgr();
			
		}
		return singleton;
	}
	
	//在断开连接或者退出网络模式时，需要调用close方法释放各种资源
	public static void close(){
		OnlineMgr mgr = getInstance();
		mgr.stop = true;;
		mgr.reciveMsgThread.interrupt();
		mgr.heartThread.interrupt();
		try {
			if(mgr.socket!=null)mgr.socket.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		singleton=null;
	}
	
	/**发送洗牌结果**/
	public static void sendCleanCards(List<Card> bottomCards, List<Player> players, Callback callback) {
		Map<Integer, List<Integer>> cards = new HashMap<Integer, List<Integer>>();
		/**封装底牌**/
		List<Integer> bottomIndexs = new ArrayList<Integer>();
		for(Card c:bottomCards) {
			bottomIndexs.add(c.getIndex());
		}
		cards.put(GameView.BOTTOM_KEY, bottomIndexs);
		/**封装每个玩家的牌**/
		for(Player p : players) {
			List<Integer> indexs = new ArrayList<Integer>();
			for(Card c : p.cards) {
				indexs.add(c.getIndex());
			}
			cards.put(p.id, indexs);
		}
		sendCmd(CmdVO.START, J.toJson(cards), G.me.id, callback);
	}
	
	public interface Callback{
		void execute(boolean success,CmdVO c);
	}

	public static void sendCmd(String cmd){
		sendCmd(cmd, null, null,null);
	}
	public static void sendCmd(String cmd, Callback callback){
		sendCmd(cmd, null, null,callback);
	}
	public static void sendCmd(String cmd,Integer playerId){
		sendCmd(cmd, null, playerId,null);
	}
	public static void sendCmd(String cmd,String content){
		sendCmd(cmd, content, null);
	}
	public static void sendCmd(String cmd,String content,Callback callback){
		sendCmd(cmd, content, null,callback);
	}
	public static void sendCmd(String cmd, Integer content ,Integer playerId){
		sendCmd(cmd, content.toString(), playerId, null);
	}
	public static void sendCmd(String cmd,String content,Integer playerId,Callback callback){
		OnlineMgr mgr = getInstance();
		StringBuilder requestCode = new StringBuilder()
									.append(System.currentTimeMillis())
									.append((int)(Math.random()*100000));
		CmdVO vo = new CmdVO(playerId, cmd, content, requestCode.toString());
		if(callback!=null) mgr.callbacks.put(requestCode.toString(), callback);
		mgr.sendData(vo);
	}
	
	private void sendData(CmdVO vo){
		try{
			cmdStart = System.currentTimeMillis();
			while(out == null) {
				if(System.currentTimeMillis()-cmdStart>CMD_OVER_TIME*1000) {
					Callback callback=callbacks.get(vo.requestCode);
					if(callback!=null){
						callback.execute(false, vo);
						callbacks.remove(vo.requestCode);
					}
					return;
				}
			}
			out.write((gson.toJson(vo)+"\r\n").getBytes());
			out.flush();
		} catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * 处理各种通用的消息
	 * @param command
	 */
	protected void handleMsg(CmdVO c){
		if(CmdVO.LIST.equals(c.cmd) || CmdVO.INFO.equals(c.cmd)) { //服务器发送桌子信息过来
			Activity room = G.activityMap.get(Room.class);
			if(room instanceof Room) {
				((Room)room).execute(true, c);
			}
		} else if(CmdVO.UPDATE.equals(c.cmd) || CmdVO.START.equals(c.cmd) 
				|| CmdVO.CALL.equals(c.cmd) || CmdVO.CARD.equals(c.cmd)
				|| CmdVO.READY.equals(c.cmd) || CmdVO.UNREADY.equals(c.cmd)
				|| CmdVO.OUTLINE.equals(c.cmd)) {
			Activity game = G.activityMap.get(Game.class);
			if(game instanceof Game) {
				((Game)game).execute(true, c);
			}
		} else if(CmdVO.FORCE_EXIT.equals(c.cmd)) {
			((Base)G.context).execute(true, c);
		}
	}
	
	//在单独的线程中运行，不断的从服务器上读取消息
	public void run() {
		try {
			if(socket==null) {
				socket = new Socket(IP, new Integer(PORT));
				socket.setSoTimeout(Time_OUT);
			}
			if(socket!=null) {
				in=socket.getInputStream();
				out=socket.getOutputStream();
				startReciveMessage();
			}
		} catch (Exception e) {
			if(e instanceof java.net.SocketTimeoutException) {
				((Base)G.context).sendMsg(Base.TIME_OUT);
			}
			e.printStackTrace();
		}
	}
	
	public void startReciveMessage() throws Exception{
		String line = null;
		BufferedReader reader = new BufferedReader(new InputStreamReader(in,"utf-8"));
		while (!stop && (line = reader.readLine()) != null) {
			CmdVO c = gson.fromJson(line, CmdVO.class);
			if(c.requestCode!=null){
				Callback callback = callbacks.get(c.requestCode);
				if(callback!=null){
					callback.execute(true, c);
					callbacks.remove(c.requestCode);
				} else {
					handleMsg(c);
				} 
			} else {
				handleMsg(c);
			}
		}
	}
}
