package rt.lanyo.game.ddz.mgr;

import java.util.Arrays;
import java.util.List;

import rt.lanyo.game.ddz.R;
import rt.lanyo.game.ddz.logic.CardStyle;
import rt.lanyo.game.ddz.model.Card;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.util.G;
import rt.lanyo.game.ddz.util.S;
import android.content.res.Resources;
/**
 * 闯关任务管理，主要负责对通关的判断
 */
public class TaskMgr {
	public static final byte SUCCESS = 1;
	public static final byte FAILED = 2;
	public static final byte ING = 3;
	
	public static final byte TYPE_ONE = 0; //以某种牌型胜利为赢,不限局数
	public static final byte TYPE_TWO = 1;//N局中赢M局
	public static final byte TYPE_THREE = 2;//连赢N局为胜利
	public static final byte TYPE_FOUR = 3;//胜利时牌中包含了某些牌为赢,不限局数
	public static final byte TYPE_FIVE = 4;//赢掉对方N分为赢,不限局数
	
	
	 /***隐藏着关卡信息***/
	private static String split;
	private static String split2;
	private static String placeholder;
	private static String n;
	private static String m;
	private static String k;
	
	public static boolean out = false;
	public static String[] tasks;
	public static String[] taskIntros;
	public static CardStyle[] styles;
	private static Resources res;
	
	static {
		res = G.context.getResources();
		if(tasks==null) tasks = res.getStringArray(R.array.tasks);
		if(taskIntros==null) taskIntros = res.getStringArray(R.array.tasks_info);
		if(styles==null) {
			styles = new CardStyle[4];
			styles[0]  = CardStyle.Single;
			styles[1]  = CardStyle.Double;
			styles[2]  = CardStyle.TripleAndOne;
			styles[3]  = CardStyle.TripleAndTwo;
		}
		
		if(split==null) split = res.getString(R.string.split);
		if(split2==null) split2 = res.getString(R.string.split2);
		if(placeholder==null) placeholder = res.getString(R.string.placeholder);
		if(n==null) n = res.getString(R.string.n);
		if(m==null) m = res.getString(R.string.m);
		if(k==null) k = res.getString(R.string.k);
	}
	
	/**
	 * 判断玩家的闯关情况
	 * @return -1:失败,继续闯关;0:闯关中;1:成功
	 */
	public static byte judge(Player player) {
		out = player.score<50;
		
		int rounds = player.rounds;
		int loseRounds = player.loseRounds;
		String task = tasks[(player.taskId==null?1:player.taskId)-1]; //查找此关的任务
		String[] types = task.split(split); //对任务进行解析
		
		if(S.valid(types[TYPE_ONE], placeholder)) { //以某种牌型胜利为赢,不限局数
			int i = Integer.parseInt(types[TYPE_ONE]);
			if(player.winner && player.lastStyle==styles[i]) {
				return SUCCESS;
			} 
			return ING;
		}
		
		if(S.valid(types[TYPE_TWO], placeholder)) { //N局中赢M局
			String[] nm = types[TYPE_TWO].split(split2);
			int n = Integer.parseInt(nm[0]);
			int m = Integer.parseInt(nm[1]);
			if(rounds<=n && loseRounds<=n-m) {
				if(rounds-loseRounds>=m) {
					return SUCCESS;
				}
				return ING;
			} else {
				return FAILED;
			}
		}
		
		if(S.valid(types[TYPE_THREE], placeholder)) { //连赢N局为胜利
			int n = Integer.parseInt(types[TYPE_THREE]);
			if(loseRounds>0) {
				player.rounds = 0; //如果输了一局,就将rounds清零
				return FAILED;
			}
			if(rounds>=n) {
				return SUCCESS;
			}
			return ING;
		}
		
		if(S.valid(types[TYPE_FOUR], placeholder)) {//胜利时牌中包含了某些牌为赢,不限局数
			List<String> list = Arrays.asList(types[TYPE_FOUR].split(split2));
			List<Card> cards = player.lastCards;
			for(Card c : cards) {
				if(list.contains(String.valueOf(c.getValue()))) {
					return SUCCESS;
				}
			}
			return ING;
		}
		
		if(S.valid(types[TYPE_FIVE], placeholder)) { //赢掉对方N分为赢,不限局数
			if(player.leftScore<=0 && player.rightScore<=0) {
				return SUCCESS;
			}
			return ING;
		}
		return ING;
	}
	
	public static int getTaskNums() {
		return tasks.length;
	}
	
	/**
	 * 是否为最后一种类型：赢光对方的N积分
	 * @param id
	 * @return
	 */
	public static int[] loadScores(Player player) {
		int[] is = new int[]{Player.INIT_CPU_SCORE, Player.INIT_CPU_SCORE}; //默认是100分
		
		if(G.gameMode!=G.GAME_MODE_TASK) return is;
		
		int taskId = player.taskId==0?1:player.taskId;
		int rounds = player.rounds;
		String ret = tasks[taskId-1].split(split)[TYPE_FIVE];
		if(S.valid(ret, placeholder)) { //判断是否为最后一种类型
			if(rounds==0) { //从string.XML文件中加载
				is[0] = Integer.parseInt(ret);
				is[1] = Integer.parseInt(ret);
			} else { //从player中获得
				is[0] = player.leftScore;
				is[1] = player.rightScore;
			}
		}
		return is;
	}
	
	public static String getTaskContent(Player player) {
		StringBuilder content = new StringBuilder(taskIntros[player.taskId-1]);
		Integer rounds = player.rounds==null?0:player.rounds;
		Integer loseRounds = player.loseRounds==null?0:player.loseRounds;
		if(content.indexOf(k) != -1){
			S.replace(content, k, rounds.toString());
		} else if(content.indexOf(n) != -1){
			S.replace(content, n, rounds.toString());
			S.replace(content, m, String.valueOf(rounds-loseRounds));
		}
		return content.toString();
	}
	
	public static byte loadMultiple(Integer taskId) {
		if(G.gameMode == G.GAME_MODE_NORMAL) return G.NORMAL_MULTIPLE;
		if(G.gameMode == G.GAME_MODE_FUN) return G.FUN_MULTIPLE;
		taskId = taskId==0?1:taskId;
		
		String s = tasks[taskId-1].substring(tasks[taskId-1].lastIndexOf(split)+1);
		return (byte)Integer.parseInt(s);
	}
	
	public static String getIntroStr(Integer taskId) {
		taskId = taskId==0?1:taskId;
		String n = tasks[taskId-1].substring(tasks[taskId-1].lastIndexOf(split)+1);
		String s = res.getString(R.string.task_intro);
		return s.replace(placeholder, n);
	}
	
}
