package rt.lanyo.game.ddz.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import rt.lanyo.game.ddz.logic.CardStyle;
import android.graphics.Bitmap;

/**
 * 游戏角色
 */
@SuppressWarnings("serial")
public class Player implements Serializable{
	public static final Integer INIT_SCORE = 0;
	public static final Integer INIT_CPU_SCORE = 100;
	
	public static final byte NO_LOCATION =-1;
	public static final byte LEFT =1 ;
	public static final byte RIGHT =2 ;
	public static final byte BOTTOM =3 ;
	
	public static final Integer LEFT_ID = -1;
	public static final Integer RIGHT_ID = -2;
	public static final byte ME_INDEX = 2;
	public static final byte LEFT_INDEX = 0;
	public static final byte RIGHT_INDEX = 1;
	
	//性别
	public static final byte MALE = 0;
	public static final byte FEMALE = 1;
	
	public static final String TABLE = "player";
	public static final String ID = "id";
	public static final String NAME = "name";
	public static final String ICON_INDEX = "iconIndex";
	public static final String TITLE = "title";
	public static final String SCORE = "score";
	public static final String SEX = "sex"; 
	public static final String ROUNDS = "rounds";
	public static final String LOSE_ROUNDS = "loseRounds";
	public static final String TASK_ID = "taskId";
	public static final String LEFT_SCORE = "leftScore";
	public static final String RIGHT_SCORE = "rightScore"; 
	public static final String ACCOUNT = "account";
	public static final String PWD = "pwd";
	
	public static final String RESULT = "result";	
	public Integer id;
	//角色名称
	public String name;
	public Integer iconIndex;
	//图标
	public Bitmap icon;
	//头衔
	public String title;
	//积分
	public Integer score;
	//是否是电脑
	public Boolean computer;
	
	//0是男,1是女
	public Integer sex;
	
	//是否为地主
	public boolean lord;
	
	public boolean ready = false;
	
	public String account;
	public String pwd;
	public Integer win;
	public Integer lose;
	public Integer escape;
	public Boolean rememberPwd;
	public Boolean autoLogin;
	public Long lastLoginTime;
	
	//头像出现的x坐标
	public int x;
	//头像出现的y坐标
	public int y;
	//消息区域的y坐标
	public int msgY;
	//消息区域的x坐标
	public int msgX;
	
	//位置,左边还是右边,0表示左上,1表示右上,2表示上 3表示下
	public Integer  position;
	
	//手上所有的牌
	public List<Card> cards = Collections.synchronizedList(new ArrayList<Card>());
	//上次出的牌
	public List<Card> lastCards = Collections.synchronizedList(new ArrayList<Card>());
	//当前要出的牌
	public List<Card> willCards = Collections.synchronizedList(new ArrayList<Card>());
	
	public List<Player> partners = Collections.synchronizedList(new ArrayList<Player>());
	
	public Player next;
	public Player privous;
	
	public Integer result; //一盘中的得分
	public boolean winner; //是否自己赢了,同伴赢了不算
	public Integer taskRounds;//从闯关开始所经历的局数,重新开始才会清零
	public Integer rounds; //从领任务开始经历的局数,过完一关就清零
	public Integer loseRounds; // 从领任务开始输的局数
	public Integer taskId; //第几关
	public Integer leftScore; //左边的分数
	public Integer rightScore; //右边的分数
	public CardStyle lastStyle; //胜利时最后出的牌型
	//决定发牌的人，在网络模式下，只能由一个客户端发牌，其它人负责接收其发的牌
	public boolean distributor;
	//桌号
	public int deskId;
	//是否桌子主人，桌子主人有权限踢人，给桌子设置密码，设定玩法等
	public boolean deskMaster;
	public void cleanCard(){
		synchronized (cards) {
			cards.clear();
		}
		synchronized (lastCards) {
			lastCards.clear();
		}
		synchronized (partners) {
			partners.clear();
		}
		synchronized (willCards) {
			willCards.clear();
		}
		lord=false;
	}
	public void commitCards(){
		commitCards(willCards);
		synchronized (willCards) {
			willCards.clear();
		}
	}
	public void commitCards(List<Card> nowCards){
		synchronized (cards) {
			cards.removeAll(nowCards);
		}
		synchronized (lastCards) {
			lastCards.clear();
			lastCards.addAll(nowCards);
		}
		
	}
	
	public Card getCard(int value,int color){
		for(Card c:cards){
			if(c.getValue()==value&&c.getColor()==color)return c;
		}
		return null;
	}
	
	public Player(String account, String pwd) {
		this.account = account;
		this.pwd = pwd;
	}
	public Player(String name, String account, String pwd,
			Integer iconIndex, Integer sex) {
		this.name = name;
		this.account = account;
		this.pwd = pwd;
		this.iconIndex = iconIndex;
		this.sex = sex;
	}
	public Player() {
	}
	public Player(String account, String pwd, Boolean rememberPwd, Boolean autoLogin) {
		this.account = account;
		this.pwd = pwd;
		this.rememberPwd = rememberPwd;
		this.autoLogin = autoLogin;
	}
}
